local Tasks = Tasks
local L = Tasks.L
local TasksUtils = TasksUtils
local db
local itemFramePool, headerFramePool, frameLookup = {}, {}, {}
local taskFrameCount, headerFrameCount = 1,1

Tasks:RegisterCallback("Tasks-Initialize", "DisplayInit")

function Tasks:DisplayInit()
  db = Tasks.db
  self:CreateTasksFrame()
  
  Tasks:RegisterCallback("Tasks-TaskCompleted", "DisplayTaskCompleted")
  Tasks:RegisterCallback("Tasks-TaskReset", "DisplayTaskReset")
end

-- self.TasksFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
--                                           --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
--                                           tile = true, tileSize = 16, edgeSize = 16, 
--                                           insets = { left = 4, right = 4, top = 4, bottom = 4 }});

function Tasks:CreateTasksFrame()
  local frameSettings = db.global.displayOptions.frameSettings
  self.TasksFrame = CreateFrame("Frame","TasksFrame",UIParent, "TasksTemplate")
  --self.TasksFrame:SetMovable(true)
  --self.TasksFrame:EnableMouse(not frameSettings.locked)
  --self.TasksFrame:SetClampedToScreen(true)
  --self.TasksFrame:RegisterForDrag("LeftButton")
  if not frameSettings.locked then Tasks.TasksFrame.button:RegisterForDrag("LeftButton")
  else Tasks.TasksFrame.button:RegisterForDrag() end
  self.TasksFrame.button:RegisterForDrag("LeftButton")
  self.TasksFrame.button:SetPushedTextOffset(0,0)
  --self.TasksFrame:SetScript("OnClick", function(frame) InterfaceOptionsFrame_OpenToCategory("Task Lists") end)
  --self.TasksFrame:SetScript("OnDragStart", function(frame) frame:StartMoving() end)
  --self.TasksFrame:SetScript("OnDragStop", function(frame) frame:StopMovingOrSizing() end)

  self.TasksFrame.button:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            });

  self.TasksFrame.button:SetBackdropColor(0,0,255,0);
  
    self.TasksFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            });

  self.TasksFrame:SetBackdropColor(255,0,255,0);
  --self.TasksFrame:SetHeight(20)
  --self.TasksFrame:SetWidth(150)
  --self.TasksFrame:SetAlpha(1.0)
  --self.TasksFrame:SetText("|cff2FE7EBTasks|r")

  --self.TasksFrame.title = self.TasksFrame:CreateFontString("TasksTitle", nil, "GameFontNormalLarge")
  --self.TasksFrame.title:SetText("|cff2FE7EBTasks|r")
  --self.TasksFrame.title:SetPoint("TOPLEFT", self.TasksFrame, "TOPLEFT", 5, -5)
  --self.TasksFrame.title:Show()

  --set a default location. if user moves it, blizzard will save to layout-local.txt and restore automatically
  self.TasksFrame:SetPoint("CENTER", UIParent, "CENTER", 300, 380)

  self:CreateTaskListFrames(self.TasksFrame)
end

function Tasks:GetItemFrame(taskId, task, listFrame)
  local tempFrame
  if table.getn(itemFramePool) > 0 then
    tempFrame = itemFramePool[1]
    table.remove(itemFramePool,1)
  else
    tempFrame = CreateFrame("Frame","TasksItemFrame"..taskFrameCount,listFrame.childrenFrame, "TasksItemTemplate")
    tempFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            })
    
    tempFrame:SetBackdropColor(255,255,0,0)
    tempFrame:SetAlpha(1.0)
    taskFrameCount = taskFrameCount+1
  end
    tempFrame.task = task
  frameLookup[taskId] = tempFrame
  return tempFrame
end
function Tasks:GetHeaderFrame(listId,list)
  local tempFrame
  if table.getn(headerFramePool) > 0 then
    tempFrame = headerFramePool[1]
    table.remove(headerFramePool,1)
  else
    tempFrame = CreateFrame("Frame","TasksHeaderFrame"..headerFrameCount,self.TasksFrame, "TasksHeaderTemplate")
    tempFrame.childrenFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            --edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            })
    
    tempFrame.childrenFrame:SetBackdropColor(255,0,0,0)
    tempFrame:SetAlpha(1.0)
    headerFrameCount = headerFrameCount+1

    tempFrame.childrenFrame.origSetHeight = tempFrame.childrenFrame.SetHeight
    function tempFrame.childrenFrame:SetHeight(newHeight)
      self:origSetHeight(newHeight)
      self:GetParent():SetHeight(newHeight+15)
    end
  end
  tempFrame.children = {}
  tempFrame.list = list
  frameLookup[listId] = tempFrame
  return tempFrame
end

function Tasks:CreateTaskListFrames(anchorFrame)
  local headerTemp
  for listId, list in pairs(db.global.lists) do
    local listHeaderFrame = Tasks:GetHeaderFrame(listId, list) --CreateFrame("Frame","TasksHeaderFrame"..listId,self.TasksFrame, "TasksHeaderTemplate")
    listHeaderFrame:SetPoint("TOPLEFT", headerTemp or anchorFrame, "BOTTOMLEFT", 0, 0)
    listHeaderFrame:Show()
    anchorFrame.nextFrame = listHeaderFrame
    listHeaderFrame.prevFrame = anchorFrame
    anchorFrame = listHeaderFrame.childrenFrame
    headerTemp = listHeaderFrame
    for taskId, task in pairs(list.tasks) do
      local taskFrame = Tasks:GetItemFrame(taskId, task, listHeaderFrame) --CreateFrame("Frame","TasksItemFrame"..taskId,listHeaderFrame.childrenFrame, "TasksItemTemplate")
      taskFrame:SetPoint("TOPLEFT", anchorFrame, listHeaderFrame.firstTask and "BOTTOMLEFT" or "TOPLEFT", 0, 0)
      taskFrame:Show()
      listHeaderFrame.firstTask = listHeaderFrame.firstTask or taskFrame
      listHeaderFrame.lastTask = taskFrame
      table.insert(listHeaderFrame.children,taskFrame)
      anchorFrame.nextFrame = taskFrame
      taskFrame.prevFrame = anchorFrame
      anchorFrame = taskFrame
      listHeaderFrame.childrenFrame:SetHeight(listHeaderFrame.childrenFrame:GetHeight()+15)
    end
    if not list.displayOptions.expanded then
      listHeaderFrame.childrenFrame:SetHeight(0)
      listHeaderFrame.childrenFrame:Hide()
    end
  end
end

function Tasks:DisplayTaskCompleted(task)
  frameLookup[task.id].checkbox:SetChecked(true)
  
  --check other tasks complete also to mark off list
  local list = db.global.lists[task.listId]
  local allDone = true;
  for _,v in pairs(list.tasks) do
    if v ~= task and not v.completedChars[self.currentChar].completed then
      allDone = false
      break;
    end
  end
  if allDone then
    frameLookup[task.listId].checkbox:SetChecked(true)
  end
end

function Tasks:DisplayTaskReset(task)
  frameLookup[task.id].checkbox:SetChecked(false)
  frameLookup[task.listId].checkbox:SetChecked(false)
end

function Tasks:TaskListClicked(checkbox)
  for k,v in ipairs(checkbox:GetParent().children) do
    if checkbox:GetChecked() then
      v.task.completedChars[self.currentChar].completed = checkbox:GetChecked()
      v.task.completedChars[self.currentChar].timestamp = time()
      v.checkbox:SetChecked(true)
    else
      v.task.completedChars[self.currentChar] = nil
      v.checkbox:SetChecked(false)
    end 
  end
end

function Tasks:TaskHeaderClicked(button)
local list = button:GetParent().list
  if list.displayOptions.expanded then
    --hide the list
    list.displayOptions.expanded = false
    _G[button:GetParent():GetName() .. "_CheckButtonText"]:SetText("|cFF76DEAF + "..list.name.."|r")
    button:GetParent().childrenFrame:SetHeight(0)
    button:GetParent().childrenFrame:Hide()
  else
    --show the list
    list.displayOptions.expanded = true
    _G[button:GetParent():GetName() .. "_CheckButtonText"]:SetText("|cFF76DEAF - "..list.name.."|r")
    button:GetParent().childrenFrame:SetHeight(table.getn(button:GetParent().children)*15)
    button:GetParent().childrenFrame:Show()
  end
end